@@ -1,7 +1,7 @@
 /*
  * pam_auth.c -- PAM authentication
  *
- * $Id: pam_auth.c,v 1.7 1997/04/05 06:53:52 morgan Exp $
+ * $Id: pam_auth.c,v 1.7 1997/04/05 06:53:52 morgan Exp morgan $
  *
  * $Log: pam_auth.c,v $
  * Revision 1.7  1997/04/05 06:53:52  morgan
@@ -46,7 +46,7 @@ int pam_setcred(pam_handle_t *pamh, int flags)
     D(("pam_setcred called"));
 
     if (! flags) {
-	flags = PAM_CRED_ESTABLISH;
+	flags = PAM_ESTABLISH_CRED;
     }
 
     retval = _pam_dispatch(pamh, flags, PAM_SETCRED);
