@@ -51,7 +51,7 @@ static int _pam_dispatch_aux(pam_handle_t *pamh, int flags, struct handler *h)
 	} else {
 	    D(("passing control to module..."));
 	    retval = h->func(pamh, flags, h->argc, h->argv);
-	    D(("module returned: %s", pam_strerror(pamh, returned)));
+	    D(("module returned: %s", pam_strerror(pamh, retval)));
 	    if (h->must_fail) {
 		D(("module poorly listed in pam.conf; forcing failure"));
 		retval = PAM_MUST_FAIL_CODE;
