@@ -19,6 +19,9 @@
 
 #include <string.h>
 #include <stdlib.h>
+#ifdef sunos
+#define memmove(x,y,z) bcopy(y,x,z)
+#endif
 
 #include "pam_private.h"
 
