@@ -61,40 +61,52 @@ char *_pam_StrTok(char *from, const char *format, char **next)
  * they are actually reached.
  */
 {
-     char table[256];
+     char table[256], *end;
      int i;
 
      if (from == NULL && (from = *next) == NULL)
 	  return from;
 
+     /* initialize table */
      for (i=1; i<256; table[i++] = '\0');
      for (i=0; format[i] ; table[(int)format[i++]] = 'y');
 
      /* look for first non-blank char */
-
      while (*from && table[(int)*from]) {
 	  ++from;
      }
 
-     if (*from) {
-	  char *end;
-
-	  /* look for next blank char */
-
-	  for (end=from; *end && !table[(int)*end]; ++end);
-
-	  if (*end)
-	       *end++ = '\0';
-
-	  if (*end) {
-	       *next = end;
-	  } else {
-	       *next = NULL;                      /* have found last token */
-	  }
+     if (*from == '[') {
+	 /*
+	  * special case, "[...]" is considered to be a single
+	  * object.  Note, however, if one of the format[] chars is
+	  * '[' this single string will not be read correctly.
+	  */
+	 for (end=++from; *end && *end != ']'; ++end) {
+	     if (*end == '\\' && end[1] == ']')
+		 ++end;
+	 }
+	 /* note, this string is stripped of its edges: "..." is what
+            remains */
+     } else if (*from) {
+	 /* simply look for next blank char */
+	 for (end=from; *end && !table[(int)*end]; ++end);
      } else {
-	  from = *next = NULL;                    /* no tokens left */
+	 return (*next = NULL);                    /* no tokens left */
      }
 
+     /* now terminate what we have */
+     if (*end)
+	 *end++ = '\0';
+
+     /* indicate what it left */
+     if (*end) {
+	 *next = end;
+     } else {
+	 *next = NULL;                      /* have found last token */
+     }
+
+     /* return what we have */
      return from;
 }
 
@@ -488,13 +500,6 @@ static void
 dopr_outch( c )
        int c;
 {
-#if 0
-       if( iscntrl(c) && c != '\n' && c != '\t' ){
-               c = '@' + (c & 0x1F);
-               if( end == 0 || output < end )
-                       *output++ = '^';
-       }
-#endif
        if( end == 0 || output < end )
                *output++ = c;
 }
@@ -551,3 +556,123 @@ void _pam_sanitize(pam_handle_t *pamh)
     pam_set_item(pamh,PAM_AUTHTOK,NULL);
     pam_set_item(pamh,PAM_OLDAUTHTOK,NULL);
 }
+
+/*
+ * This function scans the array and replaces the _PAM_ACTION_UNDEF
+ * entries with the default action.
+ */
+
+void _pam_set_default_control(int *control_array, int default_action)
+{
+    int i;
+
+    for (i=0; i<_PAM_RETURN_VALUES; ++i) {
+	if (control_array[i] == _PAM_ACTION_UNDEF) {
+	    control_array[i] = default_action;
+	}
+    }
+}
+
+/*
+ * This function is used to parse a control string.  This string is a
+ * series of tokens of the following form:
+ *
+ *               "[ ]*return_code[ ]*=[ ]*action/[ ]".
+ */
+
+#include "pam_tokens.h"
+
+void _pam_parse_control(int *control_array, char *tok)
+{
+    const char *error;
+    int ret;
+
+    while (*tok) {
+	int act, len;
+
+	/* skip leading space */
+	while (isspace(*tok) && *++tok);
+	if (!*tok)
+	    break;
+
+	/* identify return code */
+	for (ret=0; ret<=_PAM_RETURN_VALUES; ++ret) {
+	    len = strlen(_pam_token_returns[ret]);
+	    if (!strncmp(_pam_token_returns[ret], tok, len)) {
+		break;
+	    }
+	}
+	if (ret > _PAM_RETURN_VALUES || !*(tok += len)) {
+	    error = "expecting return value";
+	    goto parse_error;
+	}
+
+	/* observe '=' */
+	while (isspace(*tok) && *++tok);
+	if (!*tok || *tok++ != '=') {
+	    error = "expecting '='";
+	    goto parse_error;
+	}
+	
+	/* skip leading space */
+	while (isspace(*tok) && *++tok);
+	if (!*tok) {
+	    error = "expecting action";
+	    goto parse_error;
+	}
+
+	/* observe action type */
+	for (act=0; act<=-_PAM_ACTION_UNDEF; ++act) {
+	    len = strlen(_pam_token_actions[act]);
+	    if (!strncmp(_pam_token_actions[act], tok, len)) {
+		act *= -1;
+		tok += len;
+		break;
+	    }
+	}
+ 	if (act > 0) {
+	    /*
+	     * Either we have a number or we have hit an error.  In
+	     * principle, there is nothing to stop us accepting
+	     * negative offsets. (Although we would have to think of
+	     * another way of encoding the tokens.)  However, I really
+	     * think this would be both hard to administer and easily
+	     * cause looping problems.  So, for now, we will just
+	     * allow forward jumps.  (AGM 1998/1/7)
+	     */
+	    if (!isdigit(*tok)) {
+		error = "expecting jump number";
+		goto parse_error;
+	    }
+	    /* parse a number */
+	    act = 0;
+	    do {
+		act *= 10;
+		act += *tok - '0';      /* XXX - this assumes ascii behavior */
+	    } while (*++tok && isdigit(*tok));
+	    if (! act) {
+		/* we do not allow 0 jumps.  There is a token ('ignore')
+                   for that */
+		error = "expecting non-zero";
+		goto parse_error;
+	    }
+	}
+
+	/* set control_array element */
+	if (ret != _PAM_RETURN_VALUES) {
+	    control_array[ret] = act;
+	} else {
+	    /* set the default to 'act' */
+	    _pam_set_default_control(control_array, act);
+	}
+    }
+
+    /* that was a success */
+    return;
+
+parse_error:
+    /* treat everything as bad */
+    _pam_log_error("pam_parse: %s; [...%s]", error, tok);
+    for (ret=0; ret<_PAM_RETURN_VALUES; control_array[ret++]=_PAM_ACTION_BAD);
+
+}
