@@ -7,9 +7,17 @@
  *
  */
 
+#ifdef linux
+# define _GNU_SOURCE
+# include <features.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+/* man page says above file includes this... */
+extern int gethostname(char *name, size_t len);
+
 #include <stdarg.h>
 #include <syslog.h>
 #include <string.h>
@@ -21,6 +29,10 @@
 #include <ctype.h>
 #include <sys/utsname.h>
 
+#ifndef BROKEN_NETWORK_MATCH
+# include <netdb.h>
+# include <sys/socket.h>
+#endif
 
 /*
  * here, we make definitions for the externally accessible functions
@@ -194,10 +206,8 @@ static int list_match(char *list, struct login_info *item, match_func *match_fn)
 static char * myhostname(void)
 {
     static char name[MAXHOSTNAMELEN + 1];
-    struct utsname buf;
 
-    uname(&buf);
-    strncpy(name,buf.machine,MAXHOSTNAMELEN);
+    gethostname(name, MAXHOSTNAMELEN);
     name[MAXHOSTNAMELEN] = 0;
     return (name);
 }
@@ -281,9 +291,36 @@ static int from_match(char *tok, struct login_info *item)
     } else if (strcasecmp(tok, "LOCAL") == 0) {	/* local: no dots */
 	if (strchr(string, '.') == 0)
 	    return (YES);
+#ifdef BROKEN_NETWORK_MATCH
     } else if (tok[(tok_len = strlen(tok)) - 1] == '.'	/* network */
 	       && strncmp(tok, string, tok_len) == 0) {
 	return (YES);
+#else /*  BROKEN_NETWORK_MATCH */
+    } else if (tok[(tok_len = strlen(tok)) - 1] == '.') {
+        /*
+           The code below does a more correct check if the address specified
+           by "string" starts from "tok".
+                               1998/01/27  Andrey V. Savochkin <saw@msu.ru>
+         */
+        struct hostent *h;
+        char hn[3+1+3+1+3+1+3+1];
+        int r;
+
+        h = gethostbyname(string);
+        if (h == NULL)
+	    return (NO);
+        if (h->h_addrtype != AF_INET)
+	    return (NO);
+        if (h->h_length != 4)
+	    return (NO); /* only IPv4 addresses (SAW) */
+        r = snprintf(hn, sizeof(hn), "%u.%u.%u.%u",
+                (unsigned char)h->h_addr[0], (unsigned char)h->h_addr[1],
+                (unsigned char)h->h_addr[2], (unsigned char)h->h_addr[3]);
+        if (r < 0 || r >= sizeof(hn))
+	    return (NO);
+        if (!strncmp(tok, hn, tok_len))
+	    return (YES);
+#endif /*  BROKEN_NETWORK_MATCH */
     }
     return (NO);
 }
