@@ -1,7 +1,7 @@
 /* pam_group module */
 
 /*
- * $Id: pam_group.c,v 1.7 1997/02/15 17:31:48 morgan Exp $
+ * $Id: pam_group.c,v 1.7 1997/02/15 17:31:48 morgan Exp morgan $
  *
  * Written by Andrew Morgan <morgan@parc.power.net> 1996/7/6
  *
@@ -24,7 +24,7 @@
  */
 
 const static char rcsid[] =
-"$Id: pam_group.c,v 1.7 1997/02/15 17:31:48 morgan Exp $;\n"
+"$Id: pam_group.c,v 1.7 1997/02/15 17:31:48 morgan Exp morgan $;\n"
 "Version 0.5 for Linux-PAM\n"
 "Copyright (c) Andrew G. Morgan 1996 <morgan@parc.power.net>\n";
 
@@ -772,12 +772,12 @@ PAM_EXTERN int pam_sm_setcred(pam_handle_t *pamh, int flags
     const char *service=NULL, *tty=NULL;
     const char *user=NULL;
     int retval;
-    int setting;
+    unsigned setting;
 
     /* only interested in establishing credentials */
 
-    setting = flags & ~PAM_SILENT;
-    if (setting != PAM_CRED_ESTABLISH) {
+    setting = flags;
+    if (!(setting & PAM_ESTABLISH_CRED)) {
 	D(("ignoring call - not for establishing credentials"));
 	return PAM_SUCCESS;            /* don't fail because of this */
     }
