@@ -17,7 +17,6 @@
 
 #include <stdio.h>
 #include <unistd.h>
-#define __USE_BSD /* for bzero() prototype in string.h */
 #define __USE_POSIX2
 #include <string.h>
 #include <ctype.h>
@@ -179,9 +178,9 @@ static int is_on_group(const char *user_name, const char *group_name)
         return 0;
     if (!strlen(group_name))
         return 0;
-    bzero(uname, sizeof(uname));
+    memset(uname, 0, sizeof(uname));
     strncpy(uname, user_name, LINE_LENGTH);
-    bzero(gname, sizeof(gname));
+    memset(gname, 0, sizeof(gname));
     strncpy(gname, group_name, LINE_LENGTH);
         
     setpwent();
@@ -216,7 +215,9 @@ static int is_on_group(const char *user_name, const char *group_name)
 static int init_limits(void)
 {
     int retval = PAM_SUCCESS;
-    
+
+    D(("called."));
+
     retval |= getrlimit(RLIMIT_CPU,     &limits[RLIMIT_CPU].limit);
     limits[RLIMIT_CPU].src_soft = LIMITS_DEF_NONE;
     limits[RLIMIT_CPU].src_hard = LIMITS_DEF_NONE;
@@ -391,7 +392,7 @@ static int parse_config_file(const char *uname, int ctrl)
 #undef CONF_FILE
     
     /* init things */
-    bzero(buf, sizeof(buf));
+    memset(buf, 0, sizeof(buf));
     /* start the show */
     while (fgets(buf, LINE_LENGTH, fil) != NULL) {
         char domain[LINE_LENGTH];
@@ -417,14 +418,14 @@ static int parse_config_file(const char *uname, int ctrl)
             *tptr = '\0';
         /* Anything left ? */
         if (!strlen(buf)) {
-            bzero(buf, sizeof(buf));
+            memset(buf, 0, sizeof(buf));
             continue;
         }
 
-        bzero(domain, sizeof(domain));
-        bzero(ltype, sizeof(ltype));
-        bzero(item, sizeof(item));
-        bzero(value, sizeof(value));
+        memset(domain, 0, sizeof(domain));
+        memset(ltype, 0, sizeof(ltype));
+        memset(item, 0, sizeof(item));
+        memset(value, 0, sizeof(value));
         
         i = sscanf(buf,"%s%s%s%s", domain, ltype, item, value);
         for(j=0; j < strlen(domain); j++)
@@ -481,7 +482,9 @@ PAM_EXTERN int pam_sm_open_session(pam_handle_t *pamh, int flags,
     struct passwd *pwd;
     int ctrl;
     
-    bzero(conf_file, sizeof(conf_file));
+    D(("called."));
+
+    memset(conf_file, 0, sizeof(conf_file));
     
     ctrl = _pam_parse(argc, argv); 
     retval = pam_get_item( pamh, PAM_USER, (void*) &user_name );
@@ -553,7 +556,7 @@ struct pam_module _pam_limits_modstruct = {
 #endif
 
 /*
- * Copyright (c) Cristian Gafton, 1996, <gafton@sorosis.ro>
+ * Copyright (c) Cristian Gafton, 1996-1997, <gafton@redhat.com>
  *                                              All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
